/*-----------------------------------------------------------------------------
* FT R&D
*------------------------------------------------------------------------------
* Project     :  OAF
*
* Copyright France Telecom  [2007],  All Rights Reserved.
*
* This software is the confidential and proprietary
* information of France Telecom.
* You shall not disclose such Confidential Information
* and shall use it only in accordance with the terms
* of the license agreement you entered into with
* France Telecom.
*
*------------------------------------------------------------------------------
*
* Created     : [30/01/11]
* Last Change : [30/03/11]
* Author      : [TMT,Arnaud RINQUIN]
* Version     : [1.0b]
*
* Miscellaneous : 
*               
*------------------------------------------------------------------------------*/

(function () {
	function _OAFMashup(){
		// summary:
		//		OAFMashup class	
		
		// _location: String
		//		The based location url of the script (and proxies)
		this._location = null;
		
		// _role: String
		//		The current instance role ("parent", "child", "proxy")
		this._role = "parent";
		
		// _proxy: String
		//		The current instance proxy url
		this._proxy = null;
		
		// _handler: String
		//		The current instance handler for incomming messages
		this._handler = "onOAFMashupMessage";
		
		// _proxyUrls: Object
		//		Pointers on the Proxy URLs identified by frame ID
		this._proxyUrls = {};
		
		// _boundFrameId: String
		//		The frame ID bound to this proxy
		this._boundFrameId = null;
			
		this.init = function(){
			// summary:
			//		Initialize the OAFMashup
			
			//Parse DOC to get attributes
			var scripts = document.getElementsByTagName("script");
			var rePkg = /oafmashup.js(\W|$)/i;
			for(var i = 0; i < scripts.length; i++){
				var src = scripts[i].getAttribute("src");
				if(!src){ continue;}
				var m = src.match(rePkg);
				if(m){
					this._location = src.substring(0, m.index);
					if(scripts[i].getAttribute("role")) this._role = scripts[i].getAttribute("role");
					if(scripts[i].getAttribute("proxy")) this._proxy = scripts[i].getAttribute("proxy");
					if(scripts[i].getAttribute("handler")) this._handler = scripts[i].getAttribute("handler");
				}
			}
			
			//Set absolute path of proxy
			if(this._proxy != null && this._proxy.indexOf("http") != 0){
				if(this._proxy.indexOf("./") == 0){
					var pathLessPage = window.location.pathname.substr(0, window.location.pathname.lastIndexOf('/'));
					this._proxy = window.location.protocol + "//" + window.location.host + pathLessPage + this._proxy.substring(1);
				}
				else if(this._proxy.indexOf("/") == 0){
					this._proxy = window.location.protocol + "//" + window.location.host + this._proxy;
				}
				else{
					if(window.location.pathname.charAt(window.location.pathname.length-1) != '/') this._proxy = "/" +this._proxy;
					this._proxy = window.location.protocol + "//" + window.location.host + window.location.pathname + this._proxy;
				}		
			}
	
			//Initialize OAFMashup depending on role
			switch(this._role){
				//child -> open parent proxy if specified
				case "child" :
					if(window.location.hash.length > 2) this._createProxy(window.location.hash.substring(1),"parent");					
					window.location.hash = "#";
				break;
				//proxy -> connect resize event for message forward
				case "proxy" :
					this._boundFrameId = window.location.hash.substring(1);
					if(window.attachEvent) window.attachEvent("onfocus", function(){OAFMashup._forwardMessage();}); 	
					else this._pollLocation();
				break;
			}
		};
		
		this.open = function(/*String*/url,/*String*/id, /*String|Object*/style, /*String*/remoteProxyUrl){
			// summary:
			//		Open an overlay iFrame over current view
			//
			// url : The URL to open in iFrame
			// id : The ID used to identify the iFrame in messages
			// style : CSS className or Style object for frame layout (null/undefined for default)
			// remoteProxyUrl : Remote proxy URL to use (undefined for default, null/"" for no proxy)
	
			//Create remote proxy assuming that proxy.html is at same level than remote URL
			if(typeof(remoteProxyUrl) == "undefined")	this._createProxy(url.substring(0,url.lastIndexOf("/")+1)+"proxy.html",id);
			//Create remote proxy from specified URL (no proxy if set to null or "")
			else if(remoteProxyUrl != null && remoteProxyUrl != "") this._createProxy(remoteProxyUrl,id);
			 
			
			//Create iFrame an add it to the DOM
			var frame =document.createElement("iframe"), element;
			frame.id = id;
			frame.name = id;
			frame.allowTransparency = true;
			frame.frameBorder="0";
			
			switch(typeof(style)){
				//Class passed as style
				case "string" :
					frame.className = style;
				break;
				//Object passed as style
				case "function" :
					for(prop in style){
						if(style.hasOwnProperty(prop)) frame.style[prop] = style[prop];
					}
				break;
				//Default style
				default :
					with(frame.style){
						position = "absolute";
						left = "0px";
						top = "0px";
						width = "100%";
						height = "100%";
						visibility = "hidden";
						backgroundColor = "transparent";
					}
			}
			element = document.body.appendChild(frame);			
			frame.src = url + ((this._proxy != null) ?  "#" + this._proxy : "");
			if(frame.addEventListener) frame.addEventListener("load", function(){frame.style.visibility = "visible";},false);
			else frame.attachEvent("onload", function(){frame.style.visibility = "visible";});			
			return element;
		};
		
		this.close = function(/*String*/id){
			// summary:
			//		Close an opened frame and associated proxy
			//
			// id : The frame ID to close
			
			var pFrame = document.getElementById("_pOAFMashup"+id);
			var frame = document.getElementById(id);
			if(pFrame){
				document.body.removeChild(pFrame);
				delete pFrame;
			}
			if(frame){
				document.body.removeChild(frame);
				delete frame;
			}
		};
		
		this.sendMessage = function(/*Object*/message, /*String*/id){
			// summary:
			//		Send a message to a frame
			//
			// message : The message to send in key/value Object (one level support only)
			// id : The ID of the frame bound to proxy (null for parent)
			var pFrame;
			if(!id) id = "parent";
			pFrame = document.getElementById("_pOAFMashup"+id);
			if(pFrame){
				pFrame.contentWindow.location = this._proxyUrls[id] + '#"' + JSON.stringify(message);
				pFrame.contentWindow.focus();
			}
		};
		
		this._forwardMessage = function() {
			// summary:
			// 		Receive OUT message (from location) and forward it to bound frame
			var hash = window.location.hash.charAt(1) == '"' ? window.location.hash : decodeURIComponent(window.location.hash), // test if the double quote as been encoded
				message = JSON.parse(hash.substring(2)); // remove both hash (#) and hash encoding test double quote (")
			var frame = (this._boundFrameId == "parent") ? top : parent.frames[this._boundFrameId];
			frame[frame.OAFMashup._handler](message);
		};
		
		this._pollLocation = function(/*String*/oldLoc){
			// summary:
			// 	Fallback method used when iFrame focus is not supported by browser (Android)
			var newLoc = window.location.href+"";
			if(oldLoc && newLoc != oldLoc){
				OAFMashup._forwardMessage();
			}
			setTimeout(function(){
				try{
					OAFMashup._pollLocation(newLoc);
				}catch(e){
					//NOP, just a side effect on FF
				}
			},500);
		};
		
		this._createProxy = function(/*String*/url, /*String*/id){
			// summary:
			//		Create a proxy 
			//
			// url : The URL of the proxy
			// id : The ID of the proxy
			var pFrame =document.createElement('iframe');
			this._proxyUrls[id] = url;
			pFrame.src = url+"#"+id;
			pFrame.id = "_pOAFMashup"+id;
			pFrame.tabIndex = -1;
			with(pFrame.style){
				position = "absolute";
				left = "-100px";
				top = "0px";
				width = "1px";
				height = "1px";
			}
			return document.body.appendChild(pFrame);
		};
	};
	
	var JSON = JSON || {};
	
	JSON.stringify = JSON.stringify || function (obj) {
		// summary:
		//		Implement JSON.stringify serialization
		//
		// obj : Object to serialize
		var t = typeof (obj);
		if (t != "object" || obj === null) {
			// simple data type
			if (t == "string") obj = '"'+obj+'"';
			return String(obj);
		}
		else {
			// recurse array or object
			var n, v, json = [], arr = (obj && obj.constructor == Array);
			for (n in obj) {
				v = obj[n]; t = typeof(v);
				if (t == "string") v = '"'+v+'"';
				else if (t == "object" && v !== null) v = JSON.stringify(v);
				json.push((arr ? "" : '"' + n + '":') + String(v));
			}
			return (arr ? "[" : "{") + String(json) + (arr ? "]" : "}");
		}
	};
	
	
	// implement JSON.parse de-serialization
	JSON.parse = JSON.parse || function (str) {
		// summary:
		//		Implement JSON.parse de-serialization
		//
		// str : String to de-serialize
		if (str === "") str = '""';
		eval("var p=" + str + ";");
		return p;
	};
		
	//Initialize OAFMashup
	window.OAFMashup = new _OAFMashup();
	if(window.addEventListener) window.addEventListener("load", function(){OAFMashup.init();},false);
	else window.attachEvent("onload", function(){OAFMashup.init();});
})();	